require "import"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.content.Intent"
import "android.widget.*"
import "android.view.*"
import "android.content.Context"
import "android.text.Editable"
import "android.text.TextWatcher"

activity.setTitle("Text Analyzer")

-- Global variables
local activity = activity
local editText
local characterCountTextView, wordCountTextView, lineCountTextView, sentenceCountTextView, paragraphCountTextView

-- Function for speech recognition
local function startSpeechRecognition(language)
    local recordIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, language)

    local speechRecord = SpeechRecognizer.createSpeechRecognizer(activity)
    speechRecord.setRecognitionListener{
        onResults = function(results)
            local spokenText = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION).get(0)
            if editText then
                editText.append(spokenText .. " ")
            end
            speechRecord.destroy()
        end,
        onError = function()
            speechRecord.destroy()
            Toast.makeText(activity, "Error: Unable to recognize speech!", Toast.LENGTH_SHORT).show()
        end
    }
    speechRecord.startListening(recordIntent)
end

-- Create layout
function onCreate(savedInstanceState)
    -- Set up layout
    local scrollView = ScrollView(activity)
    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)

    -- Text input
    editText = EditText(activity)
    editText.setHint("Enter your text here")
    layout.addView(editText)

    -- Horizontal button layout for paste and speech typing
    local buttonLayout = LinearLayout(activity)
    buttonLayout.setOrientation(LinearLayout.HORIZONTAL)

    -- Paste button
    local pasteButton = Button(activity)
    pasteButton.setText("Paste")
    pasteButton.onClick = function(view)
        local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
        local clip = clipboard.getPrimaryClip()
        if clip and clip.getItemCount() > 0 then
            local pasteData = clip.getItemAt(0).getText()
            if pasteData then
                editText.setText(pasteData)
            end
        end
    end
    buttonLayout.addView(pasteButton)

    -- Hindi Speech Typing button
    local hindiSpeechButton = Button(activity)
    hindiSpeechButton.setText("Hindi Speech Typing")
    hindiSpeechButton.onClick = function(view)
        startSpeechRecognition("hi-IN") -- Hindi language code
    end
    buttonLayout.addView(hindiSpeechButton)

    -- English Speech Typing button
    local englishSpeechButton = Button(activity)
    englishSpeechButton.setText("English Speech Typing")
    englishSpeechButton.onClick = function(view)
        startSpeechRecognition("en-US") -- English language code
    end
    buttonLayout.addView(englishSpeechButton)

    -- Add button layout to main layout
    layout.addView(buttonLayout)

    -- Text counters
    characterCountTextView = TextView(activity)
    characterCountTextView.setText("Characters: 0")
    layout.addView(characterCountTextView)

    wordCountTextView = TextView(activity)
    wordCountTextView.setText("Words: 0")
    layout.addView(wordCountTextView)

    lineCountTextView = TextView(activity)
    lineCountTextView.setText("Lines: 0")
    layout.addView(lineCountTextView)

    sentenceCountTextView = TextView(activity)
    sentenceCountTextView.setText("Sentences: 0")
    layout.addView(sentenceCountTextView)

    paragraphCountTextView = TextView(activity)
    paragraphCountTextView.setText("Paragraphs: 0")
    layout.addView(paragraphCountTextView)

    scrollView.addView(layout)
    activity.setContentView(scrollView)

    -- Add TextWatcher to EditText for real-time counting
    editText.addTextChangedListener(TextWatcher {
        onTextChanged = function(s, start, before, count)
            updateCounters()
        end
    })
end

-- Function to update counters
function updateCounters()
    local text = editText.getText().toString()

    -- Character count
    local characterCount = string.len(text)
    characterCountTextView.setText("Characters: " .. characterCount)

    -- Word count
    local wordCount = 0
    for word in string.gmatch(text, "%S+") do
        wordCount = wordCount + 1
    end
    wordCountTextView.setText("Words: " .. wordCount)

    -- Line count
    local lineCount = 0
    for line in string.gmatch(text, "[^\n]*\n?") do
        lineCount = lineCount + 1
    end
    lineCountTextView.setText("Lines: " .. lineCount)

    -- Sentence count (simple approximation)
    local sentenceCount = 0
    for sentence in string.gmatch(text, "[%.!?]") do
        sentenceCount = sentenceCount + 1
    end
    sentenceCountTextView.setText("Sentences: " .. sentenceCount)

    -- Paragraph count (simple approximation)
    local paragraphCount = 0
    for paragraph in string.gmatch(text, "[^\n]*\n?") do
        if string.match(paragraph, "%S") then
            paragraphCount = paragraphCount + 1
        end
    end
    paragraphCountTextView.setText("Paragraphs: " .. paragraphCount)
end
